/*
X305Demo.c: Demo program for X305
Compiler: BC++ 3.1
          Turbo C ++ 1.01(3.01)
          
Compile mode: large

Project: X305Demo.c
         ..\..\Lib\7186el.lib
         ..\..\Lib\Xboard\X305.lib
          
Hardware: 7186EX + X305

[Nov 02, 2008] by Liam
[Dec 23, 2010] by Nicholas
[Dec, 14, 2011] by Nicholas
[March,16,2012] by Nicholas
*/
/************************************************************************/
/*  X305:   7 channel 12-Bit A/D (Analog input)  +-5.0V                 */
/*          1 channel 12-Bit D/A (Analog output) +-5.0V                 */
/*          2 channel D/I                                               */
/*          2 channel D/O                                               */
/************************************************************************/

/************************************************************************/
/*  [Software specific]                                                 */
/*      Input/Output range: -5.0V    -0.0V  +0.0V   +5.0V               */
/*      Decimal integer:    0        2047   2048    4095                */
/*      Hexadecimal:        000      7FF    800     FFF                 */
/*                                                                      */
/*      [A/D]                                                           */
/*      sampling rate:  1800 data/sec (with floating convertion)        */
/*      Accuracy==>     Typical: +- 1 LSB (+- 2.4 mV)                   */
/*                      Maximum: +- 2 LSB (+- 4.8 mV)                   */  
/*      [D/A]                                                           */
/*      thoughput :     1300 data/sec (with floating convertion)        */
/*      Accuracy==>     Typical: +- 1 LSB (+- 2.4 mV)                   */
/*                      Maximum: +- 2 LSB (+- 4.8 mV)                   */      
/*      [D/O]                                                           */
/*      throughput:  can generates 2KHz square wave signals.            */
/************************************************************************/

/************************************************************************/
/*  [Caution]                                                           */
/*      The block 7 first 16 bytes of EEPROM on X board is used to      */
/*      store A/D & D/A calibration settings. When you use the          */
/*      EEPROM on X board, don't overwrite it.                          */
/************************************************************************/

#include <stdio.h>
#include "..\..\lib\7186e.h"
#include "..\..\lib\XBoard\X305.h"

void UserTimerFunction(void);

void main(void)
{
    int iRet, iAction, quit, iFlag;
    int iValue, iInChannel, iDIValue, iOutChannel, iDOValue, iStatus;
    float fValue;
	
    InitLib();
    X305_Init();  // Initialize the X305
		
    Print("\r\n");
    Print("*****************************************\r\n");
    Print("|  Demo program for uPAC-7186EX + X305  |\r\n");
    Print("|                                       |\r\n");
    Print("|            [Nov 02, 2008]             |\r\n");
    Print("*****************************************\r\n");
        
	X305_SetDelay(5); //Set reading exchanged A/D channel delay time 0.05ms
	iFlag=0;
    while(iAction!=12)
    {
        iAction=0;
        quit=0;
        
        Puts("\n\r");
        Puts("1) Reads setting from EEPROM\n\r");
        Puts("2) A/D (Analog input)\n\r");		
        Puts("3) A/D (Analog input) read 32767 times\n\r");
        Puts("4) D/A (Analog output)\n\r");
        Puts("5) Read DI (all channels)\n\r");
        Puts("6) Read DI (one channel)\n\r");
        Puts("7) Write DO (all channels)\n\r");
        Puts("8) Write DO (one channel)\n\r");
        Puts("9) DO Readback (all channels)\n\r");
        Puts("10) DO Readback (one channel)\n\r");
		Puts("11) Start User's Timer Function to read A/D ch0, the interval is 2 sec\n\r");
		Puts("\n\r");
        Puts("12) Quits demo program\n\r\n\r");
        Puts("Choose an option and press [Enter]:");
        Scanf("%d", &iAction);
        Puts("\n\r");
        
        switch(iAction)
        {
            // Reads settings from EEPROM(on X board)
            case 1:
                iValue=X305_Init();
                if((iValue&1)==0)
                    Print("EEPROM A/D Gain ==> [%8.6f]\n\r",X305_fAD_Gain);
                else
                    Print("No setting in EEPROM, A/D Gain ==> %8.6f\n\r",X305_fAD_Gain);
                
                if((iValue&2)==0)
                    Print("EEPROM A/D Offset ==> [%8.6f]\n\r",X305_fAD_Offset);
                else
                    Print("No setting in EEPROM, A/D Offset ==> %8.6f\n\r",X305_fAD_Offset);
                
                if((iValue&4)==0)
                    Print("EEPROM D/A Gain ==> [%8.6f]\n\r",X305_fDA_Gain);
                else
                    Print("No setting in EEPROM, D/A Gain ==> %8.6f\n\r",X305_fDA_Gain);

                if((iValue&8)==0)
                    Print("EEPROM D/A Offset ==> [%8.6f]\n\r",X305_fDA_Offset);
                else
                    Print("No setting in EEPROM, D/A Offset ==> %8.6f\n\r",X305_fDA_Offset);
                
                Puts("\n\r");
            break;
            // A/D (Analog input)
            case 2:
                Puts("Please select A/D channel(0~6): ");
                Scanf("%d",&iInChannel);
                
                if(iInChannel>6)
                {
                    Puts("Unknow Channel Number!!\r\n\r\n");
                    break;
                }
                
                fValue=X305_AnalogIn(iInChannel);
                Print("AI[%d]= %6.4fV\n\r\n\r", iInChannel, fValue);
            break;
			// A/D (Analog input) reads 32767 times
            case 3:
				Puts("Please select A/D channel(0~6): ");
                Scanf("%d",&iInChannel);
                
                if(iInChannel>6)
                {
                    Puts("Unknow Channel Number!!\r\n\r\n");
                    break;
                }
                
                fValue=X305_AnalogInHexToFloat(X305_AnalogIn_no(iInChannel,32767));
                Print("AI[%d]= %6.4fV\n\r\n\r", iInChannel, fValue);
			break;
            // D/A (Analog output)
            case 4:
                Puts("Voltage (-5.0 ~ +5.0): ");
                Scanf("%f",&fValue);
                X305_AnalogOut(fValue);
                Print("AO[0]  %5.3fV OK.\n\r\n\r", fValue);
            break;
            // Read DI (all channels)
            case 5:
                iDIValue=X305_Read_All_DI();
                
                Print("Input value=> 0x%02x\r\n\r\n", iDIValue);
            break;
            // Read DI (one channel)
            case 6:
                Print("Please select inport channel(0~1): ");
                Scanf("%d", &iInChannel);
                
                if(iInChannel>1)
                {
                    Puts("Unknow Channel Number!!\r\n\r\n");
                    break;
                }
                
                iDIValue=X305_Read_One_DI(iInChannel);
                
                if(iDIValue)
                    Print("Channel %d ==> OFF\r\n\r\n", iInChannel);
                else
                    Print("Channel %d ==> ON\r\n\r\n", iInChannel);
            break;
            // Write DO (all channels)
            case 7:
                Print("Please input Output value(0 ~ 0x03): ");
                Scanf("%x", &iDOValue);
                
                X305_Write_All_DO(iDOValue);

                Print("Output value=> 0x%02x\r\n\r\n", iDOValue);
            break;
            // Write DO (one channel)
            case 8:
                Print("Please select outport channel(0~1) and status(0:OFF, 1:ON)\n\r");
                Print("Channel number and status:");
                
                Scanf("%d %d", &iOutChannel, &iStatus);
                
                if(iOutChannel>1)
                {
                    Puts("Unknow Channel Number!!\r\n\r\n");
                    break;
                }
                
                X305_Write_One_DO(iOutChannel, iStatus);
                                
                Print("Channel %d ==> %s\r\n\r\n", iOutChannel, (iStatus>0)?"ON":"OFF");
            break;
            // Read DO (all channels)
            case 9:
                iDOValue=X305_Read_All_DO();
                
                Print("Digital Output ReadBack=> 0x%02x\r\n\r\n", iDOValue);
            break;
            // Read DO (one channels)
            case 10:
                Print("Please select outport channel(0~1): ");
                Scanf("%d", &iOutChannel);
                
                if(iOutChannel>1)
                {
                    Puts("Unknow Channel Number!!\r\n\r\n");
                    break;
                }
                
                iRet=X305_Read_One_DO(iOutChannel);
                 
                if(iRet)
                    Print("Channel %d ==> ON\r\n\r\n", iOutChannel);
                else
                    Print("Channel %d ==> OFF\r\n\r\n", iOutChannel);
            break;
			//Start User's Timer Function, the interval is 2 sec
			case 11:
				if(iFlag==0)
				{
					iFlag=1;
					InstallUserTimerFunction_ms(2000,UserTimerFunction);
				}				
			break;
			
			case 12:
				if(iFlag==1)
					StopUserTimerFun();
            default:
                quit=1;
            break;
        }
        if(!quit)
        {
            Puts("Press any key to continue...\n\r");
            Getch();
        }
    }
}

void UserTimerFunction(void)
{
	int iInChannel=0;
	float fValue;
	fValue=X305_AnalogIn(iInChannel);
    Print("AI[%d]= %6.4fV\n\r\n\r", iInChannel, fValue);
}
