/*
X324Demo.c: Demo program for X324

Compiler: BC++ 3.1
          Turbo C++ 1.01(3.01) (free from http://cc.codegear.com/free/cpp)
          
Compile mode: large

Project: X324Demo.c
         ..\..\lib\7186el.lib
         ..\..\lib\XBoard\X324.lib 

Hardware: uPAC-7186EX + X324

[Nov 02, 2008] by Liam
*/

/******************************************************************/
/*  X324:   4 channel 12-Bit D/A (Analog output) 0.0~5.0V         */
/*          4 channel D/O                                         */
/******************************************************************/
/******************************************************************/
/*  [Software specific]                                           */
/*  Output range:       +0.0V   +10.0V                            */
/*  Decimal integer:    0       4095                              */
/*  Hexadecimal:        000     FFF                               */
/*                                                                */
/*  [D/A]                                                         */
/*  thoughput:  1300 data/sec (with floating convertion)          */
/*      Accuracy==>     Maximum: +/- 0.002% LSB                   */
/*                                                                */
/*  [D/O]                                                         */
/*  throughput: can generates 2KHz square wave signals.           */
/******************************************************************/
/******************************************************************/
/*  [Caution]                                                     */
/*  The EEPROM block 7 on Xboard is used to                       */
/*  store A/D & D/A calibration settings. When you use the        */
/*  EEPROM on Xboard, do not overwrite it.                        */
/******************************************************************/

#include <stdio.h>
#include "..\..\lib\7186e.h"
#include "..\..\lib\XBoard\X324.h"

void main(void)
{
    int iRet, iAction, i, quit;
    int iOutChannel, iDOValue, iStatus;
    float fValue;
    
    InitLib();
    
    iRet=X324_Init();   // Initialize the X324

    Print("\r\n");
    Print("*****************************************\r\n");
    Print("|  Demo program for uPAC-7186EX + X324  |\r\n");
    Print("|                                       |\r\n");
    Print("|            [Nov 02, 2008]             |\r\n");
    Print("*****************************************\r\n");

    if(iRet!=0)
    {
        for(i=0; i<4; i++)
        {
            if((iRet>>i*2)&0x1)
                Print("No setting in EEPROM, D/A Gain (Ch%d) ==> %+8.6f\r\n", i, X324_fDA_Gain[i]);
            
            if((iRet>>i*2+1)&0x1)
                Print("No setting in EEPROM, D/A Offset (Ch%d) ==> %+8.6f\r\n", i, X324_fDA_Offset[i]);
        }
    }
    
    while(iAction!=6)
    {
        iAction=0;
        quit=0;
        
        Puts("\r\n");
        Puts("1) D/A (Analog output)\r\n");
        Puts("2) Write DO (all channels)\r\n");
        Puts("3) Write DO (one channel)\r\n");
        Puts("4) DO Readback (all channels)\r\n");
        Puts("5) DO Readback (one channel)\r\n");
        Puts("\n\r");
        Puts("6) Quits demo program\r\n\r\n");
        Puts("Choose an option and press [Enter]:");
        Scanf("%d", &iAction);
        Puts("\r\n");
    
        switch(iAction)
        {
            // D/A (Analog output)
            case 1:
                Puts("Channel (0 ~ 3): ");
                Scanf("%d", &iOutChannel);
                
                Puts("Voltage (0.0 ~ +5.0): ");
                Scanf("%f", &fValue);
                
                X324_AnalogOut(iOutChannel, fValue);
                Print("AO[%d]  %5.3fV OK.\n\r", iOutChannel, fValue);
                break;
            
            // Write DO (all channels)
            case 2:
                Print("Please input Output value(0 ~ 0xF): ");
                Scanf("%X", &iDOValue);
                
                X324_Write_All_DO(iDOValue);

                Print("Output value=> 0x%02X\r\n\r\n", iDOValue);
                break;
            
            // Write DO (one channel)
            case 3:
                Print("Please select outport channel(0~3) and status(0:OFF, 1:ON)\r\n");
                Print("Channel number and status:");
                
                Scanf("%d %d", &iOutChannel, &iStatus);
                
                if(iOutChannel>3)
                {
                    Puts("Unknow Channel Number!!\r\n\r\n");
                    break;
                }
                
                X324_Write_One_DO(iOutChannel, iStatus);
                                
                Print("Channel %d ==> %s\r\n\r\n", iOutChannel, (iStatus>=1)?"ON":"OFF");
                break;
            
            // Read DO (all channels)
            case 4:
                iDOValue=X324_Read_All_DO();
                
                Print("Digital Output ReadBack=> 0x%02X\r\n\r\n", iDOValue);
                break;
            
            // Read DO (one channels)
            case 5:
                Print("Please select outport channel(0~3): ");
                Scanf("%d", &iOutChannel);
                
                if(iOutChannel>3)
                {
                    Puts("Unknow Channel Number!!\r\n\r\n");
                    break;
                }
                
                iRet=X324_Read_One_DO(iOutChannel);
                 
                if(iRet)
                    Print("Channel %d ==> ON\r\n\r\n", iOutChannel);
                else
                    Print("Channel %d ==> OFF\r\n\r\n", iOutChannel);
                break;
            
            case 6:
            default:
                quit=1;
                break;
        }
        
        if(!quit)
        {
            Puts("Press any key to continue...\r\n");
            Getch();
            Print("\r\n");
        }
    }
}