/*
FWDemo21.c: Web server for uPAC-7186EX series. And supports the Modbus TCP protocol.
  
Compiler: BC++ 3.1,
          Turbo C++ 1.01(3.01)

Compile mode: Large

Project: FWDemo21.c
         ..\lib\7186el.Lib
         ..\lib\tcp_dm32.lib
         ..\lib\FW_nnnnn.LIB
         
Hordware: uPAC-7186EX + X507

[Aug 27, 2010] by Liam
[Dec 23, 2010] by Nicholas
*/

#include <conio.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "..\lib\7186e.h"
#include "..\lib\Tcpip32.h"
#include "..\lib\MFW.h"
#include "..\lib\MBTCP_XS.h"

unsigned char far iMemory_DI[100];
unsigned char far iMemory_DO[100];
          int far iMemory_AI[100];
          int far iMemory_AO[100];

#define DICHANNEL   4
#define DOCHANNEL   4

//void SetupCgi(int skt);
void CGIIO(int skt); //,char *sReceive,char *sSend)
void dio(int skt);
void OS7_Dir(int skt);
void UPort502(int skt, int mode);

TCP_SERVER MBTcp={
    502,        // port
    -1,         // socket
    0,          // state
    0,          // connect
    UPort502    // CallBackFun
};

void XS_UserInit(int argc,char *argv[])
{
    extern int bAcceptBroadcast;
    extern unsigned long ACKDELAY;
    extern unsigned long MAXCDEL;
    int iRet;
  
    void UserLoop(void);

    InitLib();
    
    bAcceptBroadcast=0;
    ACKDELAY=200;
    MAXCDEL=100;

    XS_AddSystemLoopFun(UserLoop);
    
    // ======= Begin changing COM port setting =======
    // Call following codes once when you change the COM port setting
    // After saving the setting to the EEPROM, you should remove the codes
    ComData[1-1].baud=115200L;
    ComData[1-1].databit=8;
    ComData[1-1].parity=0;
    ComData[1-1].stopbit=1;
    VcomSaveComData(1-1);   // Save COM1 setting to EEPROM
    
    ComData[2-1].baud=115200L;
    ComData[2-1].databit=8;
    ComData[2-1].parity=0;
    ComData[2-1].stopbit=1;
    VcomSaveComData(2-1);   // Save COM2 setting to EEPROM
    // ======= End changing COM port setting =======
    
    // ***** Begin Modbus configuration *****
	// Before the Modbus Utility can support the 7188XB -MRTU,
	// You can add following functions to configure the Modbus settings.
	Set_NetID(1);
	
	Set_COMEnableMode(1, _Programming);
	Set_COMEnableMode(2, _Programming);
    // **** End Modbus configuration *****
    
    //======= Begin of Modbus Kernel =======
    iRet=InitModbus(iMemory_DI, iMemory_DO, iMemory_AI, iMemory_AO);
    if(iRet!=0)
    {
        // Failed to initialize Modbus lib
    }    
    Modbus_Memory_Size(100, 100, 100, 100);
    //======= End of Modbus Kernel=======
    
    // Add a Modbus server to your program
    XS_AddServer(&MBTcp);
    
    // Initialize Web server Kernal
    XS_Http_StartHttpServer();  // Add Web server
    XS_Http_SetDefaultFile("index.htm");    // Set the default response file
    XS_Http_AddCgi("dio", dio);   // Add CGI function
    XS_Http_AddCgi("cgiio", CGIIO);
    XS_Http_AddCgi("dir", OS7_Dir);
    
    // uses the TCP/IP functions
    XS_AddSystemLoopFun(XS_SocketLoopFun);
    XS_StartSocket();
    
    MAXTXTOUT=6000UL;
    EnableWDT();
}

void XS_UserEnd(void)
{
    XS_StopSocket();
    DisableWDT();
    
    RestoreCom1();
    RestoreCom2();
}

int iSendCmdProcess=0;
int SendCmdProcess(int iProcess);
void UserLoop(void)
{
    RefreshWDT();
    
    if(IsCom1())
        if(ReadCom1()=='q') QuitMain=1;
  
    iSendCmdProcess=SendCmdProcess(iSendCmdProcess);
}

char far sInputBuffer_P502[512];
void UPort502(int skt, int mode)
{
    int iByte;
    TCPREADDATA pTCP_Data_Pointer;
     
    if(mode==0)
    {
        // First time connected from remote clients
    }  
    else
    {
        iByte=readsocket(skt, sInputBuffer_P502, sizeof(sInputBuffer_P502)-1);

        if(iByte>0)
        {
            pTCP_Data_Pointer.Socket=skt;
            pTCP_Data_Pointer.ReadUartChar=sInputBuffer_P502;
            pTCP_Data_Pointer.Length=iByte;
            
            VcomCmdModbus(&pTCP_Data_Pointer);
            RefreshWDT();
        }
        else
            XS_CloseSocket(skt);
    }
}

void main(int argc, char *argv[])
{
    XS_main(argc, argv);    /* call the XS library main function. */
}

//============================//
//      Send Comand Process   //
//============================//
int SendCmdProcess(int iProcess)
{
    int i;
    unsigned ivalue;

    ivalue=0;
    switch(iProcess)
    {
        case 0:
            ivalue=inp(0);
            for(i=0; i<DICHANNEL; i++)  
                iMemory_DI[i]=(ivalue>>i) & 0x1;

            iProcess=1;
            break;
            
        case 1:
            ivalue=0;
            for(i=0; i<DOCHANNEL; i++)
            {
                if(iMemory_DO[i])
                    ivalue+=(1<<i);
            }
            outp(0, ivalue&0x3F);
            
            iProcess=0;
            break;
            
        default:
            iProcess=0;
    }
    
    return iProcess;
}

void Modbus_Request_Event(char* CommandData, int* CommandLength)
{
    CommandData=CommandData;
    CommandLength=CommandLength;   
}

void Modbus_Response_Event(char* ResponseData, int* ResponseLength)
{
    ResponseData=ResponseData;
    ResponseLength=ResponseLength;
}